# -*- coding: utf-8 -*-
#
import logging

from spf import SanicPlugin
from sanic_session import Session as RealSanicSession, InMemorySessionInterface


class Session(SanicPlugin):
    __slots__ = tuple()

    def __init__(self, *args, **kwargs):
        super(Session, self).__init__(*args, **kwargs)

    @classmethod
    def on_registered(cls, context, reg, *args, **kwargs):
        # this will need to be called more than once,
        # for every app it is registered on.
        app = context.app
        cls.init_app(app, context, *args, **kwargs)

    @classmethod
    def init_app(cls, app, context, *args, interface=None, **kwargs):
        log = context.log
        if interface is None:
            interface = InMemorySessionInterface()
        context.interface = interface
        real_session = RealSanicSession()
        real_session.interface = interface
        # Emulate old sanic-session method of attaching to app
        if not hasattr(app, 'extensions'):
            app.extensions = {}
        # session_name defaults to 'session'
        app.extensions[interface.session_name] = real_session
        log(logging.DEBUG, "Created Session named: {} on app."
            .format(interface.session_name))

instance = session = Session()


@session.middleware(attach_to="request", with_context=True)
async def add_session_to_request_context(request, context):
    interface = context.interface
    shared_context = context.shared
    shared_request_context = shared_context.request[id(request)]
    dummy_request = shared_request_context.create_child_context()
    dummy_request.cookies = request.cookies
    session_dict = await interface.open(dummy_request)
    shared_request_context[interface.session_name] = session_dict


@session.middleware(attach_to="response", with_context=True)
async def save_session(request, response, context):
    interface = context.interface
    try:
        shared_request_context = context.shared.request[id(request)]
    except KeyError as e:
        # It is possible for RESPONSE middleware to run on a 404 error _before_
        # the request middleware has run. So in this case, there is no session.
        s = int(getattr(response, 'status', 200))
        if 400 <= s <= 499:
            return
        raise
    # We dont need a dummy request or response here
    await interface.save(shared_request_context, response)




# -*- coding: utf-8 -*-
#
from sanic_session.memcache import MemcacheSessionInterface
from sanic_session.redis import RedisSessionInterface
from sanic_session.memory import InMemorySessionInterface
from sanic_session.mongodb import MongoDBSessionInterface
from sanic_session.aioredis import AIORedisSessionInterface

from sanic_session_spf.plugin import Session, session

__all__ = ('MemcacheSessionInterface', 'RedisSessionInterface',
           'InMemorySessionInterface', 'MongoDBSessionInterface',
           'AIORedisSessionInterface', 'Session', 'session')

__version__ = '0.5.1'



# -*- coding: utf-8 -*-
#
import codecs

from setuptools import setup
import re
import os


def open_local(paths, mode='r', encoding='utf8'):
    path = os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        *paths
    )
    return codecs.open(path, mode, encoding)


with open_local(['sanic_session_spf', '__init__.py'], encoding='utf-8') as fp:
    try:
        version = re.findall(r"^__version__ = '([^']+)'\r?$",
                             fp.read(), re.M)[0]
    except IndexError:
        raise RuntimeError('Unable to determine version.')

with open_local(['README.md']) as readme:
    long_description = readme.read()


with open_local(['requirements.txt']) as req:
    install_requires = req.read().split("\n")


setup(
    name='Sanic-Session-SPF',
    version=version,
    url='https://github.com/ashleysommer/sanic-session-spf',
    license='MIT',
    author='Ashley Sommer, Suby Raman',
    author_email='ashleysommer@gmail.com',
    description="A SPF compatible distribution of Sanic-Session. Wraps upstream Sanic-Session.",
    long_description=long_description,
    packages=['sanic_session_spf'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=install_requires,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
